/*
 * Created on Oct 17, 2003
 *
 */
package com.punosmusic.larrybirthday;

import java.awt.*;
import java.awt.event.*;

import com.softsynth.jsyn.*;
import com.softsynth.jmsl.*;
import com.softsynth.jmsl.jsyn.*;
import com.softsynth.jmsl.view.*;

/**
 * Birthday gift for Larry Polansky 
 * 
 * @author Nick Didkovsky, (c) 2003 Nick Didkovsky, didkovn@mail.rockefeller.edu, All rights reserved
 *
 */
public class LPBDApplet extends java.applet.Applet implements ActionListener, ItemListener {

	private boolean go;
	private static int COLS = 20;
	JMSLMixerContainer mixer;
	TextArea textArea;
	Label secretMessageLabel;
	Label distanceLabel;
	Choice choice;

	MusicShape happyBirthdayShape;
	private static String[] MSG_LIST =
		{ "LARRY", "MORPH", "CHANGE", "CONTOUR", "METRICS", "DISTANCE", "FROG PEAK", "IMPOSSIBLE", "HAPPY BIRTHDAY" };
	private String currentMsg = MSG_LIST[0];
	private String msgContourString = ContourToString.translate(CombinatorialContourAnalyzer.getNxNContour(currentMsg));

	Button goButton;
	MusicShapeEditor se;

	public void init() {
		setLayout(new BorderLayout());
	}

	public void start() {
		go = false;
		JSynMusicDevice dev = JSynMusicDevice.instance();
		dev.open();
		JMSL.clock.setAdvance(0.1);
		JSynInsFromClassName ins = new JSynInsFromClassName(8, "com.softsynth.jsyn.circuits.FilteredSawtoothBL");
		mixer = new JMSLMixerContainer();
		mixer.addInstrument(ins);
		mixer.start();
		happyBirthdayShape = new MusicShape(ins.getDimensionNameSpace());
		System.out.println(happyBirthdayShape.dimension());
		happyBirthdayShape.setRepeats(Integer.MAX_VALUE);
		buildHappyBirthdayShape();
		happyBirthdayShape.setInstrument(ins);

		se = new MusicShapeEditor();
		se.addMusicShape(happyBirthdayShape);
		add(BorderLayout.NORTH, se);

		Panel p = new Panel();
		p.setLayout(new FlowLayout());

		int longestString = 0;

		choice = new Choice();
		for (int i = 0; i < MSG_LIST.length; i++) {
			choice.add("Secret msg " + (i + 1));
			longestString = Math.max(longestString, MSG_LIST[i].length());
		}
		p.add(choice);
		choice.addItemListener(this);
		textArea = new TextArea(longestString + 2, longestString + 8);
		p.add(textArea);
		Panel labelPanel = new Panel();
		labelPanel.setLayout(new BorderLayout());

		labelPanel.add(BorderLayout.NORTH, secretMessageLabel = new Label("SECRET MESSAGE HERE"));
		labelPanel.add(BorderLayout.SOUTH, distanceLabel = new Label("DISTANCE SHOWN HERE"));
		p.add(labelPanel);
		p.add(goButton = new Button("GO"));
		goButton.addActionListener(this);
		add(BorderLayout.SOUTH, p);

		happyBirthdayShape.addRepeatPlayable(new Playable() {
			public double play(double time, Composable parent) throws InterruptedException {
				MusicShape s = (MusicShape) parent;
				MusicShape musicContour = CombinatorialContourAnalyzer.getNxNContour(happyBirthdayShape, 1);
				String musicContourString = ContourToString.translate(musicContour);
				StringScreen stringScreen = new StringScreen();
				textArea.setText(stringScreen.compare(musicContourString, msgContourString));
				double distance =
					((int) (1000 * stringScreen.getNumDifferent() / (double) musicContourString.length())) / 1000.0;
				distanceLabel.setText(distance + "");
				StringBuffer labelBuf = new StringBuffer();
				for (int i = 0; i < currentMsg.length(); i++) {
					if (CombinatorialContourAnalyzer.contourPositionsEqual(currentMsg, happyBirthdayShape, i, 1)) {
						labelBuf.append(currentMsg.charAt(i));
						happyBirthdayShape.set(JMSLRandom.choose(0.3, 0.6), i, 2);
					}
					else {
						labelBuf.append("-");
						happyBirthdayShape.set(0.0, i, 2);
					}
				}
				secretMessageLabel.setText(labelBuf.toString());
				return time;
			}
		});

	}

	private void buildHappyBirthdayShape() {
		happyBirthdayShape.removeAll();
		for (int i = 0; i < currentMsg.length(); i++) {
			double duration = 0.15;
			double pitch = JMSLRandom.choose(50.0, 74.0);
			double amplitude = 0;
			double hold = duration * 0.8;
			happyBirthdayShape.add(duration, pitch, amplitude, hold);
		}
	}

	public void stop() {
		happyBirthdayShape.finishAll();
		mixer.stop();
		JMSL.closeMusicDevices();
		removeAll();
	}

	public static void main(String[] args) {
		LPBDApplet applet = new LPBDApplet();
		AppletFrame frame = new AppletFrame("Happy Birthday Larry", applet);
		frame.setSize(800, 600);
		frame.test();
		frame.setVisible(true);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == goButton) {
			go = !go;
			if (go) {
				happyBirthdayShape.launch(JMSL.now());
				goButton.setLabel("stop");
			}
			else {
				happyBirthdayShape.finishAll();
				goButton.setLabel("go");
			}
		}

	}

	/* (non-Javadoc)
	 * @see java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
	 */
	public void itemStateChanged(ItemEvent e) {
		Object source = e.getSource();
		if (source == choice) {
			System.out.println(choice.getSelectedIndex() + "");
			currentMsg = MSG_LIST[choice.getSelectedIndex()];
			msgContourString = ContourToString.translate(CombinatorialContourAnalyzer.getNxNContour(currentMsg));
			buildHappyBirthdayShape();
			se.refresh();
		}

	}
}
